/*
 * Capability allocation and management.
 *
 * Copyright (C) 2019, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/c/util/cap_alloc.h>

#include "cap_alloc.h"
#include "semaphore.h"



/* Semaphore used if threading has been initialised by ipc_cap_alloc_init. */

l4_cap_idx_t ipc_cap_alloc_semaphore = L4_INVALID_CAP;



/* Reserve and return a capability index. */

l4_cap_idx_t ipc_cap_alloc()
{
  l4_cap_idx_t cap;
  long err;

  /* Allocate a capability in the critical section, then return. */

  err = ipc_semaphore_down(ipc_cap_alloc_semaphore);
  if (err)
    return L4_INVALID_CAP;

  cap = l4re_util_cap_alloc();

  ipc_semaphore_up(ipc_cap_alloc_semaphore);

  return cap;
}

/* Release a capability. */

int ipc_cap_free(l4_cap_idx_t cap)
{
  long err;

  err = ipc_semaphore_down(ipc_cap_alloc_semaphore);
  if (err)
    return 0;

  l4re_util_cap_free(cap);

  ipc_semaphore_up(ipc_cap_alloc_semaphore);

  return 1;
}

/* Free the indicated capability and unmap it. */

int ipc_cap_free_um(l4_cap_idx_t cap)
{
  long err;

  err = ipc_semaphore_down(ipc_cap_alloc_semaphore);
  if (err)
    return 0;

  l4re_util_cap_free_um(cap);

  ipc_semaphore_up(ipc_cap_alloc_semaphore);

  return 1;
}

// vim: tabstop=2 expandtab shiftwidth=2
